@@ -38,8 +38,8 @@ static const char *description =
   "  http://www.coova.org/\n";
 
 static const char *copyright = 
-  "Copyright (c) 2003-2005 Mondru AB., 2006-2009 Coova Technologies LLC, and others.\n"
-  "Licensed under the Gnu Public License (GPL).\n";
+  "Copyright (c) 2003-2005 Mondru AB., 2006-2010 Coova Technologies LLC, and others.\n"
+  "Licensed under the Gnu General Public License (GPL).\n";
 
 static const char *usage = \
   "Usage: chilli [OPTIONS]...\n";
@@ -74,7 +74,6 @@ char *STRDUP(char *s) {
 
 int main(int argc, char **argv) {
   struct gengetopt_args_info args_info;
-  struct options_t *opt;
   struct hostent *host;
   char hostname[USERURLSIZE];
   int numargs;
@@ -182,6 +181,7 @@ int main(int argc, char **argv) {
   _options.uamallowpost = args_info.uamallowpost_flag;
   _options.redirssl = args_info.redirssl_flag;
   _options.uamuissl = args_info.uamuissl_flag;
+  _options.domaindnslocal = args_info.domaindnslocal_flag;
 
   if (args_info.dhcpgateway_arg &&
       !inet_aton(args_info.dhcpgateway_arg, &_options.dhcpgwip)) {
@@ -321,10 +321,10 @@ int main(int argc, char **argv) {
     }
 
     if (!args_info.uamaliasname_arg ||
-	strcmp(args_info.uamaliasname_arg, hostname)) {
+	strncmp(args_info.uamaliasname_arg, hostname, strlen(args_info.uamaliasname_arg))) {
       if (!(host = gethostbyname(hostname))) {
-	log_err(0, "Could not resolve IP address of uamserver: %s! [%s]", 
-		args_info.uamserver_arg, strerror(errno));
+	log_err(0, "Could not resolve IP address of uamserver: %s!", 
+		args_info.uamserver_arg);
 	goto end_processing;
       }
       else {
@@ -352,16 +352,17 @@ int main(int argc, char **argv) {
   if (args_info.uamhomepage_arg) {
     if (get_urlparts(args_info.uamhomepage_arg, hostname, USERURLSIZE, 
 		     &_options.uamhomepageport, 0)) {
-      log_err(0,"Failed to parse uamhomepage: %s!", args_info.uamhomepage_arg);
+      log_err(0,"Failed to parse uamhomepage: %s!", 
+	      args_info.uamhomepage_arg);
       goto end_processing;
     }
 
     if (!args_info.uamaliasname_arg ||
-	strcmp(args_info.uamaliasname_arg, hostname)) {
+	strncmp(args_info.uamaliasname_arg, hostname, strlen(args_info.uamaliasname_arg))) {
       if (!(host = gethostbyname(hostname))) {
-	log_err(0,"Invalid uamhomepage: %s! [%s]", 
-		args_info.uamhomepage_arg, strerror(errno));
-	goto end_processing;
+	log_err(0,"Could not resolve uamhomepage server IP %s!", 
+		args_info.uamhomepage_arg);
+	/*goto end_processing;*/
       }
       else {
 	int j = 0;
@@ -469,7 +470,9 @@ int main(int argc, char **argv) {
 
   /* anyipexclude */
   if (args_info.anyipexclude_arg) {
-    if (option_aton(&opt->anyipexclude_addr, &opt->anyipexclude_mask, args_info.anyipexclude_arg, 0)) {
+    if (option_aton(&_options.anyipexclude_addr, 
+		    &_options.anyipexclude_mask, 
+		    args_info.anyipexclude_arg, 0)) {
       log_err(0, "Failed to parse anyip exclude network!");
       return -1;
     }
@@ -819,6 +822,9 @@ int main(int argc, char **argv) {
   if (_options.binconfig) free(_options.binconfig);
   _options.binconfig = STRDUP(args_info.bin_arg);
 
+  if (_options.ethers) free(_options.ethers);
+  _options.ethers = STRDUP(args_info.ethers_arg);
+
   ret = 0;
 
   if (_options.binconfig) { /* save out the configuration */
